/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo;


/**
 * Types for reports
 */
public enum ReportType {

    /** NDC_LIST_PRINT_TEMPLATE */
    NDC_LIST_PRINT_TEMPLATE("CaptureNDF", false, false, false, false, false),

    /** LIST_INGREDIENTS */
    LIST_INGREDIENTS("ListIngredients", false, false, false, false, false),

    /** PRINT_PRODUCTS_WARNING_LABELS */
    PRINT_PRODUCTS_WARNING_LABELS("WarningLabels", false, false, false, false, false),

    /** VA_DRUG_CLASSIFICATION_REPORT_PRINT_TEMPLATE */
    VA_DRUG_CLASSIFICATION_REPORT_PRINT_TEMPLATE("DrugClass", true, false, false, true, false),

    /** PRODUCTS_WITH_EXCLUSIONS_DRG_DRG_REPORT_PRINT_TEMPLATE */
    PRODUCTS_WITH_EXCLUSIONS_DRG_DRG_REPORT_PRINT_TEMPLATE("Exclusions", true, true, true, false, false),

    /** ACTIVE_PRDUCTS_NO_ACTIVE_NDCS_REPORT_PRINT_TEMPLATE */
    ACTIVE_PRDUCTS_NO_ACTIVE_NDCS_REPORT_PRINT_TEMPLATE("NoActiveNDCS", true, false, false, false, false),

    /** ACTIVE_PRODUCTS_PROPOSED_INACTIVATION_REPORT_PRINT_TEMPLATE */
    ACTIVE_PRODUCTS_PROPOSED_INACTIVATION_REPORT_PRINT_TEMPLATE("ProposedInactivation", true, true, false, false, false),

    /** ITEM_AUDIT_HISTORY_PRINT_TEMPLATE */
    ITEM_AUDIT_HISTORY_PRINT_TEMPLATE("VUIDApproval", true, true, false, false, false),
    
    /**
     * A ReportType representing the 'Products with Changes to Copay Tier Data' report.
     */
    PRODUCTS_WITH_CHANGES_TO_COPAY_TIER_DATA("ProductsWithChangesToCopayTierData", false, true, false, false, true);

    private String view;
    private Boolean print;
    private Boolean start;
    private Boolean stop;
    private Boolean description;
    
    /**
     * True if this ReportType should display the product status criteria section; otherwise, false.
     */
    private Boolean productStatus;

    /**
     * ReportType constructor.
     * 
     * @param pView pView
     * @param pPrint pPrint
     * @param pStart pStart
     * @param pStop pStop
     * @param pDescription pDescription
     * @param pProductStatus pProductStatus
     */
    ReportType(String pView, Boolean pPrint, Boolean pStart, Boolean pStop, Boolean pDescription, Boolean pProductStatus) {
        this.view = pView;
        this.print = pPrint;
        this.start = pStart;
        this.stop = pStop;
        this.description = pDescription;
        this.productStatus = pProductStatus;
    }

    /**
     * getDescription
     * @return description
     */
    public Boolean getDescription() {
    
        return description;
    }

    /**
     * getView
     * @return view
     */
    public String getView() {

        return this.view;
    }

    /**
     * isPrintable
     * @return print
     */
    public Boolean isPrintable() {

        return this.print;
    }

    /**
     * getSTart
     * @return start
     */
    public Boolean getStart() {
    
        return start;
    }

    /**
     * getStop
     * @return stop
     */
    public Boolean getStop() {
    
        return stop;
    }

    /**
     * Get product status.
     * 
     * @return True if the 'Product Status' criteria section should be displayed for this
     * report. Otherwise, return false.
     */
    public boolean getProductStatus() {
        return productStatus;
    }

}
