package gov.va.med.pharmacy.peps.common.vo.validator;


import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;


/**
 * CopayTier must not be null. If the CopayTier is not null, its labeler ID must not be null or empty and its name must
 * not be null or empty.
 * 
 */
public class CopayTierValidator extends AbstractValidator<CopayTierVo> {

    /**
     * Constant for max length of the Copay Tier.
     */
    private static final int MAX_COPAY_TIER_LENGTH = 3; 
    
    /**
     * CopayTier must not be null.
     * 
     * TierLevel length cannot exceed 3 characters.
     * 
     * @param target is Copay Tier VO being validated
     * 
     * @param errors list to which to add for this validation
     */
    public final void validate(CopayTierVo target, Errors errors) {
        if (isNull(target)) {
            rejectIfNull(target, errors, FieldKey.COPAY_TIER);

            return;
        }

        rejectIfNull(target.getItemStatus(), errors, FieldKey.ITEM_STATUS);
        
        if (target.getValue().length() > MAX_COPAY_TIER_LENGTH) {
            errors.addError(FieldKey.COPAY_TIER, ErrorKey.COPAY_TIER_LENGTH_EXCEEDED);
        }
        
        if ((target.getRequestItemStatus().equals(RequestItemStatus.APPROVED))) {
            FieldKey.INACTIVATION_DATE.newValidatorInstance().validate(target.getInactivationDate(), errors);
        }
    }
}
