package gov.va.med.pharmacy.peps.domain.common.capability.impl;


import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.CopayTierDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.DataAccessObject;
import gov.va.med.pharmacy.peps.domain.common.dao.EplCopayTierDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProdCopayTierAssocDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplCopayTierDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.CopayTierConverter;


/**
 * Perform CRUD operations on CopayTier.
 */
public class CopayTierDomainCapabilityImpl extends
        ManagedDataDomainCapabilityImpl<CopayTierVo, EplCopayTierDo>
        implements CopayTierDomainCapability {
    
    /**
     * The data access object for Copay Tier.
     */
    private EplCopayTierDao eplCopayTierDao;
    private EplProdCopayTierAssocDao eplProdCopayTierAssocDao;
    
    /**
     * The converter for Copay Tier.
     */
    private CopayTierConverter copayTierConverter;
    
    public final EplCopayTierDao getEplCopayTierDao() {
        return eplCopayTierDao;
    }

    
    public final void setEplCopayTierDao(EplCopayTierDao eplCopayTierDao) {
        this.eplCopayTierDao = eplCopayTierDao;
    }

    
    public final CopayTierConverter getCopayTierConverter() {
        return copayTierConverter;
    }

    
    public final void setCopayTierConverter(CopayTierConverter copayTierConverter) {
        this.copayTierConverter = copayTierConverter;
    }

    
    /**
     * deleteItem.
     * @param copayTierVo copayTierVo
     * @throws ValidationException ValidationException
     */
    @Override
    public final void deleteItem(CopayTierVo copayTierVo) throws ValidationException  {
        List<EplCopayTierDo> eplCopayTierDo = getDataAccessObject().retrieve(EplCopayTierDo.EPL_ID, 
            Long.valueOf(copayTierVo.getId()));
       
        if (eplCopayTierDo.size() == 1) {
            if (eplCopayTierDo.get(0).getTierLevel() == null) {            
                getDataAccessObject().delete(eplCopayTierDo.get(0));
            } else  {
                throw new ValidationException(ValidationException.CANNOT_DELETE,
                   copayTierVo.getId() + " because it has already synched with PRODUCT.");
            }
        }
    }

    /**
     * Create Hibernate {@link Criteria} that will find an item by its
     * uniqueness fields.
     * 
     * @param item
     *            {@link ManagedItemVo} for which to create uniqueness
     *            {@link Criteria}
     * @return {@link Criteria}
     */
    @Override
    protected final Criteria createUniquenessCriteria(CopayTierVo item) {
        Criteria criteria = getDataAccessObject().getCriteria();
        criteria.add(Restrictions.ilike(EplCopayTierDo.TIER_LEVEL,
                item.getTierLevel()));

        return criteria;
    }

    /**
     * Insert the given CopayTierVo.
     * 
     * @param drugClassVo drugClassVo
     * @param user {@link UserVo} performing the action
     * @return DrugClass with new ID
     */
    @Override
    public final CopayTierVo createWithoutDuplicateCheck(CopayTierVo copayTierVo,
            UserVo user) {

        if (copayTierVo.getId() == null) {
            copayTierVo.setId(getSeqNumDomainCapability().generateId(
                copayTierVo.getEntityType(), user));

            if ((copayTierVo.getVaDataFields() != null)
                    && (!(copayTierVo.getVaDataFields().isEmpty()))) {
                copayTierVo.getVaDataFields().setVaDfOwnerId(
                        getSeqNumDomainCapability().generatedOwnerId(user));
            }
        }

        EplCopayTierDo ingredientDo = copayTierConverter.convert(copayTierVo);

        eplCopayTierDao.insert(ingredientDo, user);


        return (copayTierVo);
    }

    /**
     * Get the {@link DataAccessObject} that this CopayTierClassCapabilityImpl uses.
     * 
     * @return {@link DataAccessObject}
     */
    public final DataAccessObject getDataAccessObject() {
        return eplCopayTierDao;
    }

    /**
     * Return the {@link Converter} instance for this capability.
     * 
     * @return {@link Converter}
     */
    public final CopayTierConverter getConverter() {
        return copayTierConverter;
    }
    
    /**
     * setEplProdCopayTierAssocDao.
     * @param eplProdCopayTierAssocDao eplProdCopayTierAssocDao property
     */
    public void setEplProdCopayTierAssocDao(
            EplProdCopayTierAssocDao eplProdCopayTiertAssocDao) {
        this.eplProdCopayTierAssocDao = eplProdCopayTierAssocDao;
    }

}
