package gov.va.med.pharmacy.peps.service.common.session;

import java.util.Date;

/**
 * This interface describes operations specific to the process which
 * generates the 'Products with Changes to Copay Tier Data' report. 
 */
public interface ReportCopayTierChangeProcess extends ReportExportProcess {
    /**
     * Get the start date for the report.
     * @return the date on or after which copay tier changes
     * should be included in the report.
     */
    public Date getStartDate();

    
    /**
     * Set the start date for the report.
     * @param startDate the date on or after which copay tier changes
     * should be included in the report.
     */
    public void setStartDate(Date startDate);

    
    /**
     * Get the flag indicating whether active products should be
     * included in the report.
     * @return true if active products are included; otherwise, false.
     */
    public boolean getIncludeActiveProducts();

    
    /**
     * Specify whether to include active products in the report.
     * 
     * @param includeActiveProducts true if active products should be
     * included; otherwise, false.
     */
    public void setIncludeActiveProducts(boolean includeActiveProducts);

    
    /**
     * Get the flag indicating whether inactive products should be
     * included in the report.
     * @return true if inactive products are included; otherwise, false.
     */
    public boolean getIncludeInactiveProducts();

    /**
     * Specify whether to include inactive products in the report.
     * 
     * @param includeInactiveProducts true if inactive products should be
     * included; otherwise, false.
     */
    public void setIncludeInactiveProducts(boolean includeInactiveProducts);

    
    /**
     * Get the value which should delimit multiple drug class codes for
     * a particular product.
     * @return the delimiter
     */
    public String getDrugClassDelimiter();

    
    /**
     * Set the value which should delimit multiple drug class codes for
     * a particular product.
     * @param drugClassDelimiter the delimiter
     */
    public void setDrugClassDelimiter(String drugClassDelimiter);
}
