package gov.va.med.pharmacy.peps.service.common.utility;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.exception.MigrationException;
import gov.va.med.pharmacy.peps.common.vo.DateFormat;
import gov.va.med.pharmacy.peps.common.vo.ReportCopayTierChangeVo;
import gov.va.med.pharmacy.peps.common.vo.ReportType;
import gov.va.med.pharmacy.peps.service.common.reports.ReportExportState;

/**
 * This class writes the data for the 'Products with Changes to Copay
 * Tier Data' report to a text file suitable for downloading.
 */
public class ReportCopayTierChangeCsvFile extends CsvFile {
    /**
     * The relative directory path for the output file.
     */
    private static final String PATH = "./tmp/";
    
    /**
     * The name of the output file.
     */
    private static final String FILENAME = ReportType.PRODUCTS_WITH_CHANGES_TO_COPAY_TIER_DATA.getView() + ".csv";
    
    /**
     * The names of the fields included in the report, in the order
     * they will be written in the header row of the output.
     */
    private static final String[] FIELD_NAMES = {
        "PRODUCT_NAME",
        "CMOP_ID",
        "NATL_FORMULARY_NAME",
        "DRUG_CLASS",
        "TIER",
        "EFFECTIVE_DATE",
        "END_DATE",
        "FUTURE_TIER_LEVEL",
        "FUTURE_EFFECTIVE_DATE",
        "STATUS_DATE"
    };
    
    /**
     * Creates file and writes data.
     * 
     * @param dataList The report data to be written to the output file.
     * @param state The ReportExportState object whose record counter will be
     * updated as rows are written to the output file.
     * @throws MigrationException if there is a problem writing the output file.
     */
    public final void createFile(List<ReportCopayTierChangeVo> dataList, ReportExportState state) throws MigrationException {
        openAndWriteHeader(PATH + FILENAME, FIELD_NAMES);
        
        SimpleDateFormat formatter = 
            new SimpleDateFormat(DateFormat.WEEKDAY_MONTH_DAY_YEAR.getFormat());
        int recordCounter = 0;
        
        for(ReportCopayTierChangeVo vo : dataList) {
            List<String> row = new ArrayList<String>();
            
            row.add(vo.getVaProductName() == null ? "" : vo.getVaProductName());
            row.add(vo.getCmopId() == null ? "" : vo.getCmopId());
            row.add(vo.getNationalFormularyName() == null ? "" : vo.getNationalFormularyName());
            row.add(vo.getDrugClass() == null ? "" : vo.getDrugClass());
            row.add(vo.getTier() == null ? "" : vo.getTier());
            row.add(vo.getEffectiveDate() == null ? "" : formatter.format(vo.getEffectiveDate()));
            row.add(vo.getEndDate() == null ? "" : formatter.format(vo.getEndDate()));
            row.add(vo.getFutureTierLevel() == null ? "" : vo.getFutureTierLevel());
            row.add(vo.getFutureEffectiveDate() == null ? "" : formatter.format(vo.getFutureEffectiveDate()));
            row.add(vo.getStatusDate() == null ? "" : formatter.format(vo.getStatusDate()));
            
            putNextRow(row.toArray(new String[row.size()]));
            
            recordCounter++;
            state.setRecordCounter(recordCounter);
        }
        
        closeExport();
    }
}
