////////////////////////////////////////////////////////////////////////////////
// checkstyle: Checks Java source code for adherence to a set of rules.
// Copyright (C) 2001-2005  Oliver Burn
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


package gov.va.med.pharmacy.checkstyle.checks;import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.FileContents;
import com.puppycrawl.tools.checkstyle.checks.AbstractFormatCheck;



public class CommentBlankLineCheck extends AbstractFormatCheck {
    /**
     * Creates a new <code>CommentBlankLineCheck</code> instance.
     */
    public CommentBlankLineCheck() {
        super("TODO:"); // the empty language
    }

    /** @see com.puppycrawl.tools.checkstyle.api.Check */
    public int[] getDefaultTokens() {
        return new int[0];
    }

    /** @see com.puppycrawl.tools.checkstyle.api.Check */
    public void beginTree(DetailAST aRootAST) {
        final FileContents contents = getFileContents();
        checkCppComments(contents);
        checkBadComments(contents);
    }

    /**
     * Checks the C++ comments for blank lines above them.
     * @param aContents the <code>FileContents</code>
     */
    private void checkCppComments(FileContents aContents) {
        final Map comments = aContents.getCppComments();
        for (final Iterator it = comments.keySet().iterator(); it.hasNext();) {
            final Integer key = (Integer) it.next();
            if ((aContents.lineIsComment(key.intValue() - 1)) && (key.intValue() - 2 >= 0)
                && (!aContents.lineIsBlank(key.intValue() - 2)) && (!aContents.lineIsComment(key.intValue() - 2))) {
                log(key.intValue(), "Need blank line above this comment (section 4)");
            }
        }
    }

    /**
     * Checks the C-style comments for blank lines above them
     * @param aContents the <code>FileContents</code>
     */
    private void checkBadComments(FileContents aContents) {
        //String[] lines = aContents.getLines();
        final Map allComments = aContents.getCComments();
        final Iterator allIter = allComments.keySet().iterator();
        while (allIter.hasNext()) {
            final Integer key = (Integer) allIter.next();
            final List lineComments = (List) allComments.get(key);
            final Iterator lineIter = lineComments.iterator();
            while (lineIter.hasNext()) {
                lineIter.next();

                if ((key.intValue() - 2 >= 0) && (!aContents.lineIsBlank(key.intValue() - 2))) {
                    log(key.intValue(), "Need blank line above block comments (section 4)");

                }
            }
        }
    }
}
