/*
 * Created on Jul 13, 2005
 *
 * $Id: MultiStatementCheck.java,v 1.1 2007/03/12 16:51:01 tstavenger Exp $
 */


package gov.va.med.pharmacy.checkstyle.checks;


import com.puppycrawl.tools.checkstyle.api.Check;
import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.TokenTypes;


/**
 * A Checkstyle Check that checks that two statements are not
 * on the same line by searching for semicolons.
 * 
 * @author njeirath
 * @version $Revision: 1.1 $ $Date: 2007/03/12 16:51:01 $
 */
public class MultiStatementCheck extends Check {

    private int lastSemiLine = -1;

    /**
     * Creates a new instance of the MultiStatementCheck
     */
    public MultiStatementCheck() {
        super();
    }

    /* (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#getDefaultTokens()
     */
    public int[] getDefaultTokens() {
        return new int[] {TokenTypes.SEMI};
    }

    /*
     *  (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#getRequiredTokens()
     */
    public int[] getRequiredTokens() {
        return getDefaultTokens();
    }

    public void beginTree(DetailAST aRootAST) {
        super.beginTree(aRootAST);
        
        this.lastSemiLine = -1;
    }

    /*
     *  (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#visitToken(com.puppycrawl.tools.checkstyle.api.DetailAST)
     */
    public void visitToken(DetailAST aAST) {

        // line numbers from DetailAST are from
        int currentLine = aAST.getLineNo();
        if ((aAST.getParent().getType() != TokenTypes.LITERAL_FOR) && (lastSemiLine == currentLine)) {
            log(aAST, "Can not have multiple statements on one line");
        }

        lastSemiLine = currentLine;
    }
}
